# -*- coding: utf-8 -*-
'''
Morphoanalysis functionality of pyvabamorf package.

Attributes
----------

PACKAGE_PATH: str
    The path where the pyvabamorf package is located.
DICT_PATH: str
    The path of the default vabamorf dictionary embedded with pyvabamorf.
phonetic_markers: str
    List of characters that make up phonetic markup.
compound_markers: str
    List of characters that make up compound markup.
phonetic_regex: regex
    Regular expression matching any phonetic marker.
compound_regex: regex
    Regular expression matching any compound marker.
'''
from __future__ import unicode_literals, print_function

import pyvabamorf.vabamorf as vm
import os
import six
import re
import operator
from functools import reduce

# path listings
PACKAGE_PATH = os.path.dirname(__file__)
DICT_PATH = os.path.join(PACKAGE_PATH, 'dct')

# various markers
phonetic_markers = frozenset('~?]<')
compound_markers = frozenset('_+=')
all_markers = phonetic_markers | compound_markers

def regex_from_markers(markers):
    '''Given a string of characters, construct a regex that matches them.
    
    Parameters
    ----------
    markers: str
        The list of string containing the markers

    Returns
    -------
    regex
        The regular expression matching the given markers.
    '''
    return re.compile('|'.join([re.escape(c) for c in markers]), flags=re.M)

phonetic_regex = regex_from_markers(phonetic_markers)
compound_regex = regex_from_markers(compound_markers)

def convert(word):
    '''This method converts given `word` to appropriate encoding and type to be given to
       SWIG wrapper.'''
    if six.PY2:
        if isinstance(word, unicode):
            return word.encode('utf-8')
        else:
            return word.decode('utf-8').encode('utf-8') # make sure it is real utf8, otherwise complain
    else: # ==> Py3
        if isinstance(word, bytes):
            return word.decode('utf-8') # bytes must be in utf8
        return word

def deconvert(word):
    '''This method converts back the output from wrapper.
       Result should be `unicode` for Python2 and `str` for Python3'''
    if six.PY2:
        return word.decode('utf-8')
    else:
        return word

def trim_phonetics(root):
    '''Function that trims phonetic markup from the root.
    
    Parameters
    ----------
    root: str
        The string to remove the phonetic markup.
        
    Returns
    -------
    str
        The string with phonetic markup removed.
    '''
    global phonetic_markers
    global phonetic_regex
    if root in phonetic_markers:
        return root
    else:
        return phonetic_regex.sub('', root)

def trim_compounds(root):
    '''Function that trims compound markup from the root.
    
    Parameters
    ----------
    root: str
        The string to remove the compound markup.
        
    Returns
    -------
    str
        The string with compound markup removed.
    '''
    global compound_markers
    global compound_regex
    
    if root in compound_markers:
        return root
    else:
        return compound_regex.sub('', root)

def get_root(root, phonetic, compound):
    '''Get the root form without markers.
    
    Parameters
    ----------
    root: str
        The word root form.
    phonetic: boolean
        If True, add phonetic information to the root forms.
    compound: boolean
        if True, add compound word markers to root forms.
    '''
    global compound_regex
    if not phonetic:
        root = trim_phonetics(root)
    if not compound:
        root = trim_compounds(root)
    return root

def get_group_tokens(root):
    '''Function to extract tokens in hyphenated groups (saunameheks-tallimeheks).
    
    Parameters
    ----------
    root: str
        The root form.
        
    Returns
    -------
    list of (list of str)
        List of grouped root tokens.
    '''
    global all_markers
    if root in all_markers or root in ['-', '_']: # special case
        return [[root]]
    groups = []
    for group in root.split('-'):
        toks = [trim_phonetics(trim_compounds(tok)) for tok in group.split('_')]
        groups.append(toks)
    return groups

def get_lemma(grouptoks, partofspeech):
    lemma = '-'.join([''.join(gr) for gr in grouptoks])
    if partofspeech == 'V' and lemma not in ['ei', 'ära']:
        lemma += 'ma'
    return lemma

def analysis_as_dict(an, trim_phonetic=True, trim_compound=True):
    '''Convert an analysis instance to a dictionary.
    Also adds "ma" ending to verbs.
    
    Parameters
    ----------
    an: vabamorf.WordAnalysis
        Analysis result as returned by vabamorf library.
    trim_phonetic: boolean
        If True, then removes phonetic annotations from root form. (default: True)
    trim_compound: boolean
        If True, then removes compund word annotations from root form. (default: True)
    
    Returns
    -------
    dict
        Morfoanalysis results.
    '''
    root = deconvert(an.root)
    
    # extract tokens and construct lemma
    grouptoks = get_group_tokens(root)
    toks = reduce(operator.add, grouptoks)
    lemma = get_lemma(grouptoks, an.partofspeech)
        
    return {'root': get_root(root, trim_phonetic, trim_compound),
            'root_tokens': toks,
            'ending': deconvert(an.ending),
            'clitic': deconvert(an.clitic),
            'partofspeech': deconvert(an.partofspeech),
            'form': deconvert(an.form),
            'lemma': lemma}

def get_args(**kwargs):
    '''Parse arguments from keyword parameters.
    
    Raises
    ------
    Exception
        In case an illegal argument is passed.
    
    Returns
    -------
    (boolean, boolean, boolean)
        Use heuristics, clean phonetics, clean compound.
    '''
    guess = True
    phonetic = True
    compound = True
    for key, value in kwargs.items():
        if key == 'guess':
            guess = bool(value)
        elif key == 'phonetic':
            phonetic = bool(value)
        elif key == 'compound':
            compound = bool(value)
        else:
            raise Exception('Unkown argument: {0}'.format(key))
    return (guess, phonetic, compound)
            
                
class PyVabamorf(object):
    '''Class for performing main tasks of morphological analysis.
    
    Attributes
    ----------
    pid: int
        Current process id.
    morf: PyVabamorf
        instance of the PyVabamorf class.
    '''

    def __init__(self, lexPath=DICT_PATH):
        '''Initialize PyVabamorf class.
        
        NB! Do not use this class directly. Instead use
        PyVabamorf.instance() to obtain access to one.
        '''
        self._analyzer = vm.Analyzer(convert(lexPath))
        self._synthesizer = vm.Synthesizer(convert(lexPath))

    @staticmethod
    def instance():
        '''Return an PyVabamorf instance.
        
        It returns the previously initialized instance or creates a new
        one if nothing exists. Also creates new instance in case the
        process has been forked.
        '''
        if not hasattr(PyVabamorf, 'pid') or PyVabamorf.pid != os.getpid():
            PyVabamorf.pid = os.getpid()
            PyVabamorf.morf = PyVabamorf()
        return PyVabamorf.morf

    def analyze(self, words, **kwargs):
        '''Perform morphological analysis on input.
        
        Parameters
        ----------
        words: list of str or str
            Either a list of pretokenized words or a string. In case of a string, it will be splitted using
            default behaviour of string.split() function.
        
        Keyword parameters
        ------------------
        guess: boolean
            If True, then use guessing, when analyzing unknown words (default: True)
        phonetic: boolean
            If True, add phonetic information to the root forms (default: True).
        compound: boolean
            if True, add compound word markers to root forms (default: True)

        Returns
        -------
        list of (list of dict)
            List of analysis for each word in input. One word usually contains more than one analysis as the
            analyser does not perform disambiguation.
        '''
        guess, phonetic, compound = get_args(**kwargs)
        
        # if input is a string, then tokenize it with whitespace
        if isinstance(words, six.string_types):
            words = words.split()
        
        # convert words to native string
        words = [convert(w) for w in words]
        
        # perform morphological analysis
        morfresult = self._analyzer.analyze(vm.StringVector(words), guess)
        result = []
        for word, analysis in morfresult:
            analysis = [analysis_as_dict(an, phonetic, compound) for an in analysis]
            result.append({'text': deconvert(word),
                           'analysis': analysis})
        return result
        
    def synthesize(self, lemma, partofspeech='', form='', hint='', guess=True, phonetic=True):
        '''Given lemma, pos tag and a form, synthesize the word.
        
        Parameters
        ----------
        lemma: str
            The lemma of the word to be synthesized.
            
        Keyword parameters
        ------------------
        partofspeech: str
            The POS tag of the word to be synthesized.
        form: str
            The form of the word to be synthesized.
        hint: str
            The hint used by vabamorf to synthesize the word.
        guess: bool
            If True, use guessing for unknown words (default: True)
        phonetic: bool
            If True, add phonetic markers to synthesized words (default: True).
            
        Returns
        -------
        list of str
            The list of synthesized words.
        '''
        words = self._synthesizer.synthesize(convert(lemma),
                                             convert(partofspeech),
                                             convert(form),
                                             convert(hint),
                                             guess,
                                             phonetic)
        return [deconvert(word) for word in words]


def analyze(words, **kwargs):
    '''Perform morphological analysis on input.
    
    Parameters
    ----------
    words: list of str or str
        Either a list of pretokenized words or a string. In case of a string, it will be splitted using
        default behaviour of string.split() function.
    
    Keyword parameters
    ------------------
    guess: boolean
        If True, then use guessing, when analyzing unknown words (default: True)
    phonetic: boolean
        If True, add phonetic information to the root forms (default: True).
    compound: boolean
        if True, add compound word markers to root forms (default: True)

    Returns
    -------
    list of (list of dict)
        List of analysis for each word in input. One word usually contains more than one analysis as the
        analyser does not perform disambiguation.
    '''
    return PyVabamorf.instance().analyze(words, **kwargs)


def synthesize(lemma, **kwargs):
    '''Given lemma, pos tag and a form, synthesize the word.

    Parameters
    ----------
    lemma: str
        The lemma of the word to be synthesized.
        
    Keyword parameters
    ------------------
    partofspeech: str
        The POS tag of the word to be synthesized.
    form: str
        The form of the word to be synthesized.
    hint: str
        The hint used by vabamorf to synthesize the word.
    guess: bool
        If True, use guessing for unknown words (default: True)
    phonetic: bool
        If True, add phonetic markers to synthesized words (default: True).
        
    Returns
    -------
    list of str
        The list of synthesized words.
    '''
    return PyVabamorf.instance().synthesize(lemma, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pyvabamorf import analyze
from multiprocessing import Pool
import unittest


class MultithreadingTest(unittest.TestCase):
    
    def test_multi(self):
        self.assertListEqual(self.compute_multi(), self.compute_single())
    
    def compute_single(self):
        return [analyze(text) for text in self.indata()]
        
    def compute_multi(self):
        pool = Pool(3)
        return pool.map(analyze, self.indata())
    
    def indata(self):
        return [
            '«Olen saatnud klubile paberi, et homseni (tänaseni – toim) on neil aega ära maksta kõik võlgnevused, mida on 4,5 kuu palga jagu. Lisaks peaksid nad mind taas kaasama esindusmeeskonna tegemistesse,» rääkis Pareiko Volgale esitatud nõudmistest ning avaldas, mis juhtub, kui neid ei täideta: «Siis katkestan teisipäeval omalt poolt meeskonnaga lepingu ning tulen kolmapäeval Tallinna. Kogu juhtum läheb seejärel edasi UEFAsse.»',
            '«Arvan, et nad ei hakka maksma ning seega on minu jaoks asjad tegelikult selged. Meeskonnas tahetakse, et eraldi treenivad mehed lahkuksid, ja praegu on asjad niimoodi ka minemas,» selgitas 37-aastane väravavaht. «Minu eest ajab asju üks Bulgaariast pärit advokaat. Tema teeb kõik tööd ära ning minu ülesanne on panna vajalikesse kohtadesse allkiri.»',
            'Pareiko on ka varasemalt avaldanud, et mõlemale osapoolele on selge, et tema vanuses jalgpallurit on raske müüa ning mänguaja leidmiseks on vajalik praegune kontraht lõpetada ja leida talvel uus tööandja. Volgaga ühepoolselt lepingu lõpetamise korral ongi vaja asjad ka UEFAga korda ajada, et Eesti koondislane saaks seejärel liituda mõne teise klubiga. Samas on vaja dokumente sellegi jaoks, et oma välja teenitud töötasu ikkagi kätte saada.',
            'Pareiko jaoks on juba plaan paigas, mis saab edasi, kui ta sel nädalal Volga meeskonna juurest lahkub. Kuna 12. novembril on Eesti koondisel kavas maavõistlusmäng Norraga ja 15. novembril EM-valikmatš San Marinoga, on väravavaht leidnud võimaluse enda vormis hoidmiseks.',
            '«27. oktoobril on plaanis minna kümneks päevaks Poola, seal on mul kokkulepe tuttava väravavahtide treeneriga. Saaksin seal harjutada ja vormi hoida. Seejärel vaataksin, kas oleks võimalik veel Levadia trennidega liituda,» tutvustas Pareiko edasisi plaane.',
            'Venemaa esiliigas hoiab veel mullu kõrgliigas mänginud Volga praegu 15. vooru järel 21 punktiga 11. positsiooni. Eile viigistati 1:1 Pareiko kunagise tööandja Tomski Tomiga. Tomsk hoiab parasjagu 29 silmaga 2. kohta ehk on praeguse seisuga püüdmas taas pääset kõrgemasse seltskonda.',
            '23-aastane India jalgpallur Peter Biaksangzuala suri pühapäeva hommikul haiglas vigastuse tagajärjel, mille ta sai viis päeva varem väravat tähistades.',
            'Biaksangzuala vigastas teisipäeval seljaaju, kui üritas väravat tähistada Miroslav Klose moodi kahekordse saltoga.',
            'Biaksangzuala klubi Bethelem Venghtlang kaotas mängu Chanmari Westile 2:3, hukkunud pallur lõi värava 62. minutil.',
            '"Ta üritas teha sakslasele Klosele omast saltoga trikki, kuid see läks kahjuks valesti," ütles üks juhtunut pealt näinud allikas reporteritele.',
            'Brasiilia karastusjookide tootja Guaraná poolt USA-s läbi viidud kampaania võitjal oli valida, kas 10 000 dollarit auhinnaraha või eksklusiivne kokkusaamine Brasiilia vutitähe Neymariga.',
            'Kampaania võitjaks osutunud 16-aastane jalgpalliga tegelev El Paso koolitüdruk Rhiannon Conelley valis kohtumise Neymariga. "Minek Hispaaniasse ja kohtumine iidoliga, see kõlab uskumatult," rääkis õnnelik Conelley kohalikule ajalehele El Paso Times. "Soovin Neymariga nii väga kohtuda ja see on mulle palju tähtsam kui 10 000 dollarit. Paljud võivad mitte nõustuda, kuid mul on õnneks võimalus ise otsustada." Ka tüdruku vanemad olid ebameeldivalt üllatunud, et kopsakas rahaline preemia jääb välja võtmata.',
            'Ajakirjanik uuris õnnelikult tüdrukult, mida ta kohtumisel Neymariga vutitähelt kindlasti küsiks. "Küsin, kas ta ei tahaks minuga abielluda," vastas Conelley naljatledes. "Loodan teda mitte hirmutada, ma ei taha, et turvamehed mind ruumist välja viskaksid," lisas tüdruk õhinal.',
            'Kampaanias osalemiseks tuli saata foto, mis oleks seotud nii jalgpalli kui Guaraná karastusjookidega. Pildid pandi firma kodulehele üles ja inimesed andsid neile hääli. Conelley pilt osutus populaarseimaks kogudes üle 4000 hääle.']

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import unittest
import operator
from pyvabamorf import analyze
from pyvabamorf.morf import trim_phonetics, get_group_tokens, analysis_as_dict, convert, deconvert
from pyvabamorf.vabamorf import Analysis
from pprint import pprint
from functools import reduce

class TrimPhoneticsTest(unittest.TestCase):
    
    def test_tilde(self):
        self.assertEqual(trim_phonetics('~'), '~')
        
    def test_questionmark(self):
        self.assertEqual(trim_phonetics('?'), '?')
    
    def test_rbracket(self):
        self.assertEqual(trim_phonetics(']'), ']')
        
    def test_lessthan(self):
        self.assertEqual(trim_phonetics('<'), '<')
        
    def test_phonetics(self):
        self.assertEqual(trim_phonetics('~~???ab]c<d'), 'abcd')


class TokensTest(unittest.TestCase):

    def get_tokens(self, root):
        return reduce(operator.add, get_group_tokens(root))

    def test_simple(self):
        tokens = self.get_tokens('all_+maa_raud_tee_jaam~~+')
        self.assertListEqual(tokens, ['all', 'maa', 'raud', 'tee', 'jaam'])

    def test_hyphen_grouping(self):
        tokens = self.get_tokens('saunameheks-tallimeheks-majahoidjaks')
        self.assertListEqual(tokens, ['saunameheks', 'tallimeheks', 'majahoidjaks'])

    def test_underscore(self):
        tokens = self.get_tokens('_')
        self.assertListEqual(tokens, ['_'])
        
    def test_plus(self):
        tokens = self.get_tokens('+')
        self.assertListEqual(tokens, ['+'])
        
    def test_equalmark(self):
        tokens = self.get_tokens('=')
        self.assertListEqual(tokens, ['='])
    
    def test_ltmark(self):
        tokens = self.get_tokens('<')
        self.assertListEqual(tokens, ['<'])
   
    def test_bracketclose(self):
        tokens = self.get_tokens(']')
        self.assertListEqual(tokens, [']'])
        
    def test_hyphen(self):
        tokens = self.get_tokens('-')
        self.assertListEqual(tokens, ['-'])
        

class AnalysisAsDictTest(unittest.TestCase):
    
    def test_verb_phonetic_compound(self):
        self.assertDictEqual(analysis_as_dict(self.verbanalysis(), True, True),
                             self.verb_phonetic_compound())
    
    def test_verb_compound(self):
        self.assertDictEqual(analysis_as_dict(self.verbanalysis(), False, True),
                             self.verb_compound())
    
    def test_verb_phonetic(self):
        self.assertDictEqual(analysis_as_dict(self.verbanalysis(), True, False),
                             self.verb_phonetic())
    
    def test_verb(self):
        self.assertDictEqual(analysis_as_dict(self.verbanalysis(), False, False),
                             self.verb())
                             
    def test_verb_default_args(self):
        self.assertDictEqual(analysis_as_dict(self.verbanalysis()),
                             self.verb_phonetic_compound())

    def verbanalysis(self):
        return Analysis(convert('l<aul'),
                        convert('b'),
                        convert(''),
                        convert('V'),
                        convert('b'))
    
    def verb_phonetic_compound(self):
        return {'clitic': '',
                'ending': 'b',
                'form': 'b',
                'lemma': 'laulma',
                'partofspeech': 'V',
                'root_tokens': ['laul'],
                'root': 'l<aul'}
    
    def verb_compound(self):
        return {'clitic': '',
                'ending': 'b',
                'form': 'b',
                'lemma': 'laulma',
                'partofspeech': 'V',
                'root_tokens': ['laul'],
                'root': 'laul'}
                
    def verb_phonetic(self):
        return {'clitic': '',
                'ending': 'b',
                'form': 'b',
                'lemma': 'laulma',
                'partofspeech': 'V',
                'root_tokens': ['laul'],
                'root': 'l<aul'}

    def verb(self):
        return {'clitic': '',
                'ending': 'b',
                'form': 'b',
                'lemma': 'laulma',
                'partofspeech': 'V',
                'root_tokens': ['laul'],
                'root': 'laul'}

    def substantiveanalysis(self):
        return Analysis(convert('lennuki_k<an]dja'),
                        convert('ile'),
                        convert(''),
                        convert('S'),
                        convert('pl all'))

    def test_substantive_phonetic_compound(self):
        self.assertDictEqual(analysis_as_dict(self.substantiveanalysis(), True, True),
                             self.substantive_phonetic_compound())
    
    def test_substantive_compound(self):
        self.assertDictEqual(analysis_as_dict(self.substantiveanalysis(), False, True),
                             self.substantive_compound())
    
    def test_substantive_phonetic(self):
        self.assertDictEqual(analysis_as_dict(self.substantiveanalysis(), True, False),
                             self.substantive_phonetic())
    
    def test_substantive(self):
        self.assertDictEqual(analysis_as_dict(self.substantiveanalysis(), False, False),
                             self.substantive())
                             
    def test_substantive_default_args(self):
        self.assertDictEqual(analysis_as_dict(self.substantiveanalysis()),
                             self.substantive_phonetic_compound())

    def substantive_phonetic_compound(self):
        return {'clitic': '',
                'ending': 'ile',
                'form': 'pl all',
                'lemma': 'lennukikandja',
                'partofspeech': 'S',
                'root_tokens': ['lennuki', 'kandja'],
                'root': 'lennuki_k<an]dja'}

    def substantive_compound(self):
        return {'clitic': '',
                'ending': 'ile',
                'form': 'pl all',
                'lemma': 'lennukikandja',
                'partofspeech': 'S',
                'root_tokens': ['lennuki', 'kandja'],
                'root': 'lennuki_kandja'}
                
    def substantive_phonetic(self):
        return {'clitic': '',
                'ending': 'ile',
                'form': 'pl all',
                'lemma': 'lennukikandja',
                'partofspeech': 'S',
                'root_tokens': ['lennuki', 'kandja'],
                'root': 'lennukik<an]dja'}
                
    def substantive(self):
        return {'clitic': '',
                'ending': 'ile',
                'form': 'pl all',
                'lemma': 'lennukikandja',
                'partofspeech': 'S',
                'root_tokens': ['lennuki', 'kandja'],
                'root': 'lennukikandja'}
                
                
class TextIsSameAsListTest(unittest.TestCase):
    
    def test_same(self):
        self.run_test(False, False, False)
        
    def test_same_guess(self):
        self.run_test(True, False, False)
    
    def test_same_phonetic(self):
        self.run_test(False, True, False)
    
    def test_same_compound(self):
        self.run_test(False, False, True)
    
    def test_same_heuristic_phonetic(self):
        self.run_test(True, True, False)

    def test_same_heuristic_compound(self):
        self.run_test(True, False, True)
        
    def test_same_phonetic_compound(self):
        self.run_test(False, True, True)
    
    def run_test(self, guess, phonetic, compound):
        text_output = analyze(self.text(),
                              guess=guess,
                              phonetic=phonetic,
                              compound=compound )
        list_output = analyze(self.text().split(),
                              guess=guess,
                              phonetic=phonetic,
                              compound=compound )
        self.assertListEqual(text_output, list_output)
    
    def text(self):
        # http://luuletused.ee/1005/elu/hetke-volu
        return '''Ma tahaks suudelda päikesekiiri 
                vat nii ihalevad mu huuled päikese suule.. 
                Ma tahaks tantsida kesksuvises vihmas 
                vat nii tunglevad minu tunded südames suures 

                Keegi kord ütles, et ära karda 
                armastus saab aja jooksul ainult kasvada 
                Võta kõik vastu, mis sulle pakutakse, 
                sest see, mis sulle antakse 
                on sinu jaoks loodud'''


                
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from pyvabamorf import synthesize
import unittest

class TestSynthesize(unittest.TestCase):
    
    def test_pood(self):
        words = synthesize('pood', form='sg p', partofspeech='S', phonetic=False)
        self.assertListEqual(words, ['poodi'])

    def test_pooma(self):
        words = synthesize('pooma', form='ti', partofspeech='V', phonetic=False)
        self.assertListEqual(words, ['poodi'])
        
    def test_kaitse(self):
        words = synthesize('kaitse', form='sg g', phonetic=False)
        self.assertListEqual(words, ['kaitse', 'kaitsme'])
        
    def test_palk(self):
        words = synthesize('palk', form='sg kom', phonetic=False, guess=False)
        self.assertListEqual(words, ['palgaga', 'palgiga'])
        
    def test_palk2(self):
        words = synthesize('palk', form='sg kom', hint='palga', phonetic=False)
        self.assertListEqual(words, ['palgaga'])

if __name__ == '__main__':
    unittest.main()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_vabamorf', [dirname(__file__)])
        except ImportError:
            import _vabamorf
            return _vabamorf
        if fp is not None:
            try:
                _mod = imp.load_module('_vabamorf', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _vabamorf = swig_import_helper()
    del swig_import_helper
else:
    import _vabamorf
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


def _swig_setattr_nondynamic_method(set):
    def set_attr(self,name,value):
        if (name == "thisown"): return self.this.own(value)
        if hasattr(self,name) or (name == "this"):
            set(self,name,value)
        else:
            raise AttributeError("You cannot add attributes to %s" % self)
    return set_attr


class SwigPyIterator(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _vabamorf.delete_SwigPyIterator
    __del__ = lambda self : None;
    def value(self): return _vabamorf.SwigPyIterator_value(self)
    def incr(self, n=1): return _vabamorf.SwigPyIterator_incr(self, n)
    def decr(self, n=1): return _vabamorf.SwigPyIterator_decr(self, n)
    def distance(self, *args): return _vabamorf.SwigPyIterator_distance(self, *args)
    def equal(self, *args): return _vabamorf.SwigPyIterator_equal(self, *args)
    def copy(self): return _vabamorf.SwigPyIterator_copy(self)
    def next(self): return _vabamorf.SwigPyIterator_next(self)
    def __next__(self): return _vabamorf.SwigPyIterator___next__(self)
    def previous(self): return _vabamorf.SwigPyIterator_previous(self)
    def advance(self, *args): return _vabamorf.SwigPyIterator_advance(self, *args)
    def __eq__(self, *args): return _vabamorf.SwigPyIterator___eq__(self, *args)
    def __ne__(self, *args): return _vabamorf.SwigPyIterator___ne__(self, *args)
    def __iadd__(self, *args): return _vabamorf.SwigPyIterator___iadd__(self, *args)
    def __isub__(self, *args): return _vabamorf.SwigPyIterator___isub__(self, *args)
    def __add__(self, *args): return _vabamorf.SwigPyIterator___add__(self, *args)
    def __sub__(self, *args): return _vabamorf.SwigPyIterator___sub__(self, *args)
    def __iter__(self): return self
SwigPyIterator_swigregister = _vabamorf.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)


def cdata(*args):
  return _vabamorf.cdata(*args)
cdata = _vabamorf.cdata

def memmove(*args):
  return _vabamorf.memmove(*args)
memmove = _vabamorf.memmove
class StringVector(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    def iterator(self): return _vabamorf.StringVector_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _vabamorf.StringVector___nonzero__(self)
    def __bool__(self): return _vabamorf.StringVector___bool__(self)
    def __len__(self): return _vabamorf.StringVector___len__(self)
    def pop(self): return _vabamorf.StringVector_pop(self)
    def __getslice__(self, *args): return _vabamorf.StringVector___getslice__(self, *args)
    def __setslice__(self, *args): return _vabamorf.StringVector___setslice__(self, *args)
    def __delslice__(self, *args): return _vabamorf.StringVector___delslice__(self, *args)
    def __delitem__(self, *args): return _vabamorf.StringVector___delitem__(self, *args)
    def __getitem__(self, *args): return _vabamorf.StringVector___getitem__(self, *args)
    def __setitem__(self, *args): return _vabamorf.StringVector___setitem__(self, *args)
    def append(self, *args): return _vabamorf.StringVector_append(self, *args)
    def empty(self): return _vabamorf.StringVector_empty(self)
    def size(self): return _vabamorf.StringVector_size(self)
    def clear(self): return _vabamorf.StringVector_clear(self)
    def swap(self, *args): return _vabamorf.StringVector_swap(self, *args)
    def get_allocator(self): return _vabamorf.StringVector_get_allocator(self)
    def begin(self): return _vabamorf.StringVector_begin(self)
    def end(self): return _vabamorf.StringVector_end(self)
    def rbegin(self): return _vabamorf.StringVector_rbegin(self)
    def rend(self): return _vabamorf.StringVector_rend(self)
    def pop_back(self): return _vabamorf.StringVector_pop_back(self)
    def erase(self, *args): return _vabamorf.StringVector_erase(self, *args)
    def __init__(self, *args): 
        this = _vabamorf.new_StringVector(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _vabamorf.StringVector_push_back(self, *args)
    def front(self): return _vabamorf.StringVector_front(self)
    def back(self): return _vabamorf.StringVector_back(self)
    def assign(self, *args): return _vabamorf.StringVector_assign(self, *args)
    def resize(self, *args): return _vabamorf.StringVector_resize(self, *args)
    def insert(self, *args): return _vabamorf.StringVector_insert(self, *args)
    def reserve(self, *args): return _vabamorf.StringVector_reserve(self, *args)
    def capacity(self): return _vabamorf.StringVector_capacity(self)
    __swig_destroy__ = _vabamorf.delete_StringVector
    __del__ = lambda self : None;
StringVector_swigregister = _vabamorf.StringVector_swigregister
StringVector_swigregister(StringVector)

class AnalysisVector(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    def iterator(self): return _vabamorf.AnalysisVector_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _vabamorf.AnalysisVector___nonzero__(self)
    def __bool__(self): return _vabamorf.AnalysisVector___bool__(self)
    def __len__(self): return _vabamorf.AnalysisVector___len__(self)
    def pop(self): return _vabamorf.AnalysisVector_pop(self)
    def __getslice__(self, *args): return _vabamorf.AnalysisVector___getslice__(self, *args)
    def __setslice__(self, *args): return _vabamorf.AnalysisVector___setslice__(self, *args)
    def __delslice__(self, *args): return _vabamorf.AnalysisVector___delslice__(self, *args)
    def __delitem__(self, *args): return _vabamorf.AnalysisVector___delitem__(self, *args)
    def __getitem__(self, *args): return _vabamorf.AnalysisVector___getitem__(self, *args)
    def __setitem__(self, *args): return _vabamorf.AnalysisVector___setitem__(self, *args)
    def append(self, *args): return _vabamorf.AnalysisVector_append(self, *args)
    def empty(self): return _vabamorf.AnalysisVector_empty(self)
    def size(self): return _vabamorf.AnalysisVector_size(self)
    def clear(self): return _vabamorf.AnalysisVector_clear(self)
    def swap(self, *args): return _vabamorf.AnalysisVector_swap(self, *args)
    def get_allocator(self): return _vabamorf.AnalysisVector_get_allocator(self)
    def begin(self): return _vabamorf.AnalysisVector_begin(self)
    def end(self): return _vabamorf.AnalysisVector_end(self)
    def rbegin(self): return _vabamorf.AnalysisVector_rbegin(self)
    def rend(self): return _vabamorf.AnalysisVector_rend(self)
    def pop_back(self): return _vabamorf.AnalysisVector_pop_back(self)
    def erase(self, *args): return _vabamorf.AnalysisVector_erase(self, *args)
    def __init__(self, *args): 
        this = _vabamorf.new_AnalysisVector(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _vabamorf.AnalysisVector_push_back(self, *args)
    def front(self): return _vabamorf.AnalysisVector_front(self)
    def back(self): return _vabamorf.AnalysisVector_back(self)
    def assign(self, *args): return _vabamorf.AnalysisVector_assign(self, *args)
    def resize(self, *args): return _vabamorf.AnalysisVector_resize(self, *args)
    def insert(self, *args): return _vabamorf.AnalysisVector_insert(self, *args)
    def reserve(self, *args): return _vabamorf.AnalysisVector_reserve(self, *args)
    def capacity(self): return _vabamorf.AnalysisVector_capacity(self)
    __swig_destroy__ = _vabamorf.delete_AnalysisVector
    __del__ = lambda self : None;
AnalysisVector_swigregister = _vabamorf.AnalysisVector_swigregister
AnalysisVector_swigregister(AnalysisVector)

class WordAnalysis(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    def __init__(self, *args): 
        this = _vabamorf.new_WordAnalysis(*args)
        try: self.this.append(this)
        except: self.this = this
    first = _swig_property(_vabamorf.WordAnalysis_first_get, _vabamorf.WordAnalysis_first_set)
    second = _swig_property(_vabamorf.WordAnalysis_second_get, _vabamorf.WordAnalysis_second_set)
    def __len__(self): return 2
    def __repr__(self): return str((self.first, self.second))
    def __getitem__(self, index): 
      if not (index % 2): 
        return self.first
      else:
        return self.second
    def __setitem__(self, index, val):
      if not (index % 2): 
        self.first = val
      else:
        self.second = val
    __swig_destroy__ = _vabamorf.delete_WordAnalysis
    __del__ = lambda self : None;
WordAnalysis_swigregister = _vabamorf.WordAnalysis_swigregister
WordAnalysis_swigregister(WordAnalysis)

class SentenceAnalysis(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    def iterator(self): return _vabamorf.SentenceAnalysis_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self): return _vabamorf.SentenceAnalysis___nonzero__(self)
    def __bool__(self): return _vabamorf.SentenceAnalysis___bool__(self)
    def __len__(self): return _vabamorf.SentenceAnalysis___len__(self)
    def pop(self): return _vabamorf.SentenceAnalysis_pop(self)
    def __getslice__(self, *args): return _vabamorf.SentenceAnalysis___getslice__(self, *args)
    def __setslice__(self, *args): return _vabamorf.SentenceAnalysis___setslice__(self, *args)
    def __delslice__(self, *args): return _vabamorf.SentenceAnalysis___delslice__(self, *args)
    def __delitem__(self, *args): return _vabamorf.SentenceAnalysis___delitem__(self, *args)
    def __getitem__(self, *args): return _vabamorf.SentenceAnalysis___getitem__(self, *args)
    def __setitem__(self, *args): return _vabamorf.SentenceAnalysis___setitem__(self, *args)
    def append(self, *args): return _vabamorf.SentenceAnalysis_append(self, *args)
    def empty(self): return _vabamorf.SentenceAnalysis_empty(self)
    def size(self): return _vabamorf.SentenceAnalysis_size(self)
    def clear(self): return _vabamorf.SentenceAnalysis_clear(self)
    def swap(self, *args): return _vabamorf.SentenceAnalysis_swap(self, *args)
    def get_allocator(self): return _vabamorf.SentenceAnalysis_get_allocator(self)
    def begin(self): return _vabamorf.SentenceAnalysis_begin(self)
    def end(self): return _vabamorf.SentenceAnalysis_end(self)
    def rbegin(self): return _vabamorf.SentenceAnalysis_rbegin(self)
    def rend(self): return _vabamorf.SentenceAnalysis_rend(self)
    def pop_back(self): return _vabamorf.SentenceAnalysis_pop_back(self)
    def erase(self, *args): return _vabamorf.SentenceAnalysis_erase(self, *args)
    def __init__(self, *args): 
        this = _vabamorf.new_SentenceAnalysis(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args): return _vabamorf.SentenceAnalysis_push_back(self, *args)
    def front(self): return _vabamorf.SentenceAnalysis_front(self)
    def back(self): return _vabamorf.SentenceAnalysis_back(self)
    def assign(self, *args): return _vabamorf.SentenceAnalysis_assign(self, *args)
    def resize(self, *args): return _vabamorf.SentenceAnalysis_resize(self, *args)
    def insert(self, *args): return _vabamorf.SentenceAnalysis_insert(self, *args)
    def reserve(self, *args): return _vabamorf.SentenceAnalysis_reserve(self, *args)
    def capacity(self): return _vabamorf.SentenceAnalysis_capacity(self)
    __swig_destroy__ = _vabamorf.delete_SentenceAnalysis
    __del__ = lambda self : None;
SentenceAnalysis_swigregister = _vabamorf.SentenceAnalysis_swigregister
SentenceAnalysis_swigregister(SentenceAnalysis)


def FSCInit():
  return _vabamorf.FSCInit()
FSCInit = _vabamorf.FSCInit

def FSCTerminate():
  return _vabamorf.FSCTerminate()
FSCTerminate = _vabamorf.FSCTerminate
class Analysis(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    root = _swig_property(_vabamorf.Analysis_root_get, _vabamorf.Analysis_root_set)
    ending = _swig_property(_vabamorf.Analysis_ending_get, _vabamorf.Analysis_ending_set)
    clitic = _swig_property(_vabamorf.Analysis_clitic_get, _vabamorf.Analysis_clitic_set)
    partofspeech = _swig_property(_vabamorf.Analysis_partofspeech_get, _vabamorf.Analysis_partofspeech_set)
    form = _swig_property(_vabamorf.Analysis_form_get, _vabamorf.Analysis_form_set)
    def __init__(self, *args): 
        this = _vabamorf.new_Analysis(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _vabamorf.delete_Analysis
    __del__ = lambda self : None;
Analysis_swigregister = _vabamorf.Analysis_swigregister
Analysis_swigregister(Analysis)

class Analyzer(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    def __init__(self, *args): 
        this = _vabamorf.new_Analyzer(*args)
        try: self.this.append(this)
        except: self.this = this
    def analyze(self, *args): return _vabamorf.Analyzer_analyze(self, *args)
    __swig_destroy__ = _vabamorf.delete_Analyzer
    __del__ = lambda self : None;
Analyzer_swigregister = _vabamorf.Analyzer_swigregister
Analyzer_swigregister(Analyzer)

class Synthesizer(object):
    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    def __init__(self, *args): 
        this = _vabamorf.new_Synthesizer(*args)
        try: self.this.append(this)
        except: self.this = this
    def synthesize(self, *args): return _vabamorf.Synthesizer_synthesize(self, *args)
    __swig_destroy__ = _vabamorf.delete_Synthesizer
    __del__ = lambda self : None;
Synthesizer_swigregister = _vabamorf.Synthesizer_swigregister
Synthesizer_swigregister(Synthesizer)






# -*- coding: utf-8 -*-
import pyvabamorf.vabamorf as vm
import atexit

if not vm.FSCInit():
    raise Exception('Could not initiate pyvabamorf library. FSCInit() returned false!')

@atexit.register
def terminate():
    vm.FSCTerminate()

from morf import analyze, synthesize
from morf import PyVabamorf



# -*- coding: utf-8 -*-
from setuptools import setup, Extension
import os
import sys

def get_sources(src_dir='src', ending='.cpp'):
    '''Function to get a list of files ending with `ending` in `src_dir`.'''
    return [os.path.join(src_dir, fnm) for fnm in os.listdir(src_dir) if fnm.endswith(ending)]

# define directories for vabamorf source directories
dirs = ['fsc', 'proof', 'etana', 'json']
src_dirs = [os.path.join('src', d) for d in dirs]

# define a list of C++ source files
lib_sources = []
for d in src_dirs:
    lib_sources.extend(get_sources(d))

# define directories for vabamorf include directories
dirs.append(os.path.join('fsc', 'fsjni'))
include_dirs = [os.path.join('include', d) for d in dirs]

# define the vabamorf SWIG wrapper generator interface file
swig_interface = os.path.join('pyvabamorf', 'vabamorf.i')
swig_opts = ['-c++', '-modern']

# Python 3 specific configuration
extra = {}
if sys.version_info[0] == 3:
    swig_opts.append('-py3')

setup(name='pyvabamorf',
    version="1.6",
    description='Python interface for the Vabamorf Estonian lemmatizer and morphological analyzer.',
    author='Tarmo Vaino, Heiki-Jaan Kaalep, Sven Laur, Timo Petmanson, Aleksandr Tkachenko, Siim Orasmaa, Raul Sirel',
    author_email='tpetmanson@gmail.com',
    url='https://github.com/brainscauseminds/pyvabamorf',
    classifiers = ['Intended Audience :: Developers',
                   'Intended Audience :: Education',
                   'Intended Audience :: Science/Research',
                   'Intended Audience :: Information Technology',
                   'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
                   'Operating System :: OS Independent',
                   'Topic :: Scientific/Engineering',
                   'Topic :: Scientific/Engineering :: Artificial Intelligence',
                   'Topic :: Scientific/Engineering :: Information Analysis',
                   'Topic :: Text Processing',
                   'Topic :: Text Processing :: Linguistic'],

    use_2to3=True,

    ext_modules = [
        Extension('pyvabamorf._vabamorf',
                  [swig_interface] + lib_sources,
                  swig_opts = swig_opts,
                  include_dirs=include_dirs)
        ],

    packages = ['pyvabamorf', 'pyvabamorf.tests'],
    package_dir = {'pyvabamorf': 'pyvabamorf',
                   'pyvabamorf.test': os.path.join('pyvabamorf', 'tests')},
    package_data = {'pyvabamorf': [os.path.join('dct', 'et.dct')]}
    )
